@@ -1,4 +1,4 @@
-from kombu import Connection, Exchange, Queue
+from kombu import Connection, Queue, Exchange
 
 from .event import Event
 from ..core import _settings
@@ -12,19 +12,25 @@ def dispatch(event: Event, amqp_name: str = ''):
         raise Exception('Settings not set:sub config of AMQP_CONFIGS not set')
     if 'AMQP_SERVER_ADDRESS' not in amqp_config:
         raise Exception('Settings not set: AMQP_SERVER_ADDRESS of sub config')
-    routing_key = amqp_config.get(
-        'ROUTING_KEY'
-    ) or f"{_settings.BALI_ROUTING_KEY}_{event.type}"
+
+    routing_key = None
+    exchange_type = amqp_config.get('EXCHANGE_TYPE')
     exchange = Exchange(
-        amqp_config.get('EXCHANGE_NAME', _settings.BALI_EXCHANGE),
-        type=amqp_config.get('EXCHANGE_TYPE')
-    )
-    queue = Queue(
-        amqp_config.get('QUEUE_NAME') or
-        f"{ _settings.BALI_QUEUE}_{event.type}",
-        exchange,
-        routing_key=routing_key
+        name=amqp_config.get('EXCHANGE_NAME') or _settings.BALI_EXCHANGE,
+        type=exchange_type
     )
+    declare = [exchange]
+    if exchange_type != 'fanout':
+        routing_key = amqp_config.get(
+            'ROUTING_KEY'
+        ) or _settings.BALI_ROUTING_KEY.format(event.type)
+    if amqp_config.get('QUEUE_NAME'):
+        queue = Queue(
+            amqp_config.get('QUEUE_NAME'),
+            exchange,
+            routing_key=routing_key
+        )
+        declare.append(queue)
     with Connection(amqp_config['AMQP_SERVER_ADDRESS']) as conn:
         # produce
         producer = conn.Producer(serializer='json')
@@ -32,6 +38,6 @@ def dispatch(event: Event, amqp_name: str = ''):
             event.dict(),
             exchange=exchange,
             routing_key=routing_key,
-            declare=[queue],
+            declare=declare,
             retry=True
         )
