@@ -79,7 +79,7 @@ def handle():
                 queues=[items[0].queue],
                 accept=['json'],
                 callbacks=[i.callback for i in items],
-                auto_declare=False
+                auto_declare=True
             ) as consumer:
                 try:
                     conn.drain_events(timeout=2)
