@@ -20,22 +20,22 @@ class ModelResource(Resource):
 
     @action()
     def list(self, schema_in: ListRequest = None):
-        return self.model.query().filter(
+        return self.model.io.query().filter(
             *get_filters_expr(self.model, **schema_in.filters)
         )
 
     @action()
     def get(self, pk=None):
-        return self.model.first(id=pk)
+        return self.model.io.first(id=pk)
 
     @action()
     def create(self, schema_in: schema = None):
         # noinspection PyUnresolvedReferences
-        return self.model.create(**schema_in.dict())
+        return self.model.io.create(**schema_in.dict())
 
     @action()
     def update(self, schema_in: schema = None, pk=None):
-        item = self.model.first(id=pk)
+        item = self.model.io.first(id=pk)
         # noinspection PyUnresolvedReferences
         for k, v in schema_in.dict().items():
             if v is None:
@@ -45,6 +45,6 @@ class ModelResource(Resource):
 
     @action()
     def delete(self, pk=None):
-        item = self.model.first(id=pk)
+        item = self.model.io.first(id=pk)
         item.delete()
         return {'id': pk, 'result': True}
