@@ -5,23 +5,22 @@ A Resource layer base class to handle FastAPI and gRPC requests and responses
 Resource's method input is Pydantic schema
 
 """
-import inspect
-import logging
-import typing
-from collections import OrderedDict
-from typing import Optional, Callable
 
-from fastapi import Request, HTTPException, status
+from collections import OrderedDict
+from typing import Optional
+
+import decamelize
+from fastapi import HTTPException, status
 from fastapi_pagination import LimitOffsetPage
 from google.protobuf import message
 from pydantic import BaseModel
 
-from ..paginate import paginate
-from ..routing import APIRouter
-from ..schemas import ListRequest, ResultResponse
 from .generic_routes import *
+from .._utils import pluralize
+from ..routing import APIRouter
+from ..schemas import ResultResponse, model_to_schema
 
-__all__ = ['Resource', 'GENERIC_ACTIONS']
+__all__ = ['GENERIC_ACTIONS', 'Resource', 'pre_process']
 
 GENERIC_ACTIONS = [
     'list',
@@ -32,7 +31,33 @@ GENERIC_ACTIONS = [
 ]
 
 
-class Resource:
+# noinspection PyUnresolvedReferences
+class ResourceMeta(type):
+    @property
+    def _http_endpoint(self):
+        if self.http_endpoint:
+            endpoint = self.http_endpoint
+        else:
+            # Generate endpoint from resource name
+            name = self.__name__.replace('Resource', '')
+            words = decamelize.convert(name).split('_')
+            words[-1] = pluralize(words[-1])
+            endpoint = '-'.join(words)
+
+        if not endpoint.startswith('/'):
+            endpoint = f'/{endpoint}'
+        return endpoint
+
+    def _get_rpc_object(self, action):
+        """The object being manipulated"""
+        name = self.__name__.replace('Resource', '')
+        if action == 'list':
+            name = pluralize(name)
+
+        return name
+
+
+class Resource(metaclass=ResourceMeta):
     """Base Resource
 
     Generic Actions is get, list, create, update, delete
@@ -40,6 +65,10 @@ class Resource:
 
     # Resource's name, should automatic recognition is not provided
     name = None
+    plural_name = None
+
+    # http_endpoint is `prefix` args for FastAPI include_router
+    http_endpoint = None
 
     schema = None
     filters = []
@@ -57,16 +86,48 @@ class Resource:
 
         self.auth = BaseModel()
 
+    # noinspection PyMethodFirstArgAssignment
     @classmethod
     def as_router(cls):
+        cls = pre_process(cls)
         return RouterGenerator(cls)()
 
+    # noinspection PyMethodFirstArgAssignment
+    @classmethod
+    def as_servicer(cls, app):
+        return ServicerGenerator(cls)(app)
+
+
+def pre_process(resource_cls):
+    """Preprocess Resource"""
+
+    # Auto generate model schema
+    if resource_cls.schema is None:
+        if getattr(resource_cls, 'model', None) is not None:
+            resource_cls.schema = model_to_schema(
+                resource_cls.model,
+                partial=True,
+            )
+
+    return resource_cls
+
+
+# noinspection PyUnresolvedReferences
+class Generator:
+    @property
+    def resource_name(self):
+        return self.cls.__name__.replace('Resource', '')
+
+    @property
+    def primary_key(self):
+        return 'id'
+
 
 # noinspection PyShadowingBuiltins,PyUnresolvedReferences,PyProtectedMember
 # noinspection PyShadowingNames,DuplicatedCode
-class RouterGenerator:
+class RouterGenerator(Generator):
     """
-    Generator router from Resource class
+    Generator FastAPI router from Resource class
     """
     def __init__(self, cls):
         self.cls = cls
@@ -89,14 +150,6 @@ class RouterGenerator:
             self.add_route(action, extra)
         return self.router
 
-    @property
-    def resource_name(self):
-        return self.cls.__name__.replace('Resource', '')
-
-    @property
-    def primary_key(self):
-        return 'id'
-
     def check_permissions(self, resource):
         for permission_class in self.cls.permission_classes:
             permission = permission_class(resource)
@@ -257,3 +310,51 @@ class RouterGenerator:
                 methods=methods,
                 summary=f"{action.replace('_', ' ')}"
             )
+
+
+# noinspection PyProtectedMember
+class ServicerGenerator(Generator):
+    """
+    Generator gRPC servicer from Resource class
+    """
+    def __init__(self, cls):
+        self.cls = cls
+
+    def __call__(self, app):
+        self.app = app
+        actions = sorted(
+            self.cls._actions.keys(), key=lambda x: x in GENERIC_ACTIONS
+        )
+
+        # noinspection PyProtectedMember
+        for action in actions:
+            extra = self.cls._actions.get(action)
+            if not hasattr(self.cls, action):
+                continue
+            self.add_servicer(action, extra)
+
+    def add_servicer(self, action, extra):
+
+        response_pb = None
+        method = f'{action.capitalize()}{self.cls._get_rpc_object(action)}'
+
+        if action == 'list':
+            response_pb = self.app.pb2.ListResponse
+        elif action == 'create':
+            response_pb = self.app.pb2.ItemResponse
+        elif action == 'get':
+            response_pb = self.app.pb2.ItemResponse
+        elif action == 'update':
+            response_pb = self.app.pb2.ItemResponse
+        elif action == 'delete':
+            response_pb = self.app.pb2.ResultResponse
+
+        def servicer(_, request, context):
+            resource = self.cls(
+                request,
+                context,
+                response_pb,
+            )
+            return getattr(resource, action)()
+
+        setattr(self.app._rpc_servicer, method, servicer)
